/* Copyright (c) 2015-present, salesforce.com, inc. All rights reserved
  Licensed under BSD 3-Clause - see LICENSE.txt or git.io/sfdc-license */

import { fragmentFrom } from 'elix/src/core/htmlLiterals';
import { firstRender, render, setState, state, template } from 'elix/src/base/internal';
import { standardBooleanAttributes } from 'elix/src/core/AttributeMarshallingMixin';
import SdsInputCheckbox from '@salesforce-ux/sds-input-checkbox/input-checkbox';
import { reflectAttribute } from '@salesforce-ux/sds-utils';

import html from './input-checkbox.html';
import css from './input-checkbox.module.css';

/**
 * > FOR TESTING ONLY <
 *
 * This component represents the native shadow DOM half of mixed-mode. Mixed-mode
 * currently requires markup and CSS to be the same in both modes.
 */

/**
 * Extend Elix's standard boolean attributes definition
 */
Object.assign(standardBooleanAttributes, {
  invalid: true,
});
export class SldsInputCheckbox extends SdsInputCheckbox {
  get invalid() {
    return this[state].invalid;
  }

  set invalid(invalid) {
    return this[setState]({ invalid });
  }

  get required() {
    return this[state].required;
  }

  set required(required) {
    return this[setState]({ required });
  }

  /**
   * Handle internal 'change' event triggered by checkbox
   */
  _handleChange(checkbox) {
    super._handleChange(checkbox);
    const inputCheckbox = this.shadowRoot.querySelector('[part~="input-checkbox"]');

    /**
     * Sync our states with the IDL properties of the checkbox
     */
    this[setState]({ invalid: !checkbox.validity.valid });

    this._handleInvalid(inputCheckbox);
  }

  /**
   * Handle invalid state of checkbox
   *
   * This is a mock validation implementation for the sole purpose of mimicking
   * LBC validation behavior and provding styling examples.
   */
  _handleInvalid(inputCheckbox) {
    const invalid = this[state].invalid;
    const helpText = this.shadowRoot.querySelector('[part~="help-text"]');
    const helpTextEl = `<div id="help-message" role="alert" part="help-text">Complete this field.</div>`;

    if (invalid && !helpText) {
      inputCheckbox.insertAdjacentHTML('afterend', helpTextEl);
    }

    if (!invalid && helpText) {
      helpText.remove();
    }
  }

  _handleRequired(checkbox) {
    const required = this[state].required;
    const requiredMarker = this.shadowRoot.querySelector('[title="required"]');
    const requiredMarkerEl = `<abbr class="slds-required" title="required">*</abbr>`;

    if (required && !requiredMarker) {
      checkbox.insertAdjacentHTML('beforebegin', requiredMarkerEl);
    }

    if (!required && requiredMarker) {
      required.remove();
    }
  }

  [render](changed) {
    super[render](changed);
    const { invalid } = this[state];
    const inputCheckbox = this.shadowRoot.querySelector('[part~="input-checkbox"]');
    const checkbox = this.shadowRoot.querySelector('[part~="checkbox"]');

    if (this[firstRender] && invalid) {
      this._handleInvalid(inputCheckbox);
    }

    if (changed.invalid) {
      reflectAttribute(this, 'invalid', invalid);
    }

    if (changed.required) {
      this._handleRequired(checkbox);
    }
  }

  /**
   * Blow away SDS markup, we need LBC markup structure here. But we decorate with
   * `part` to align with the sds-input-checkbox's spec. We also don't completely
   * remove the SDS template since we still want a dynamic link to the styles.
   */
  get [template]() {
    const result = super[template];
    result.content.querySelector('[part~="input-checkbox"]').remove();
    result.content.append(fragmentFrom.html`
      <style>
        ${css}
      </style>
      ${html}
    `);

    return result;
  }
}
